import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/models/ChatModels.dart';

import 'ChatPage.dart';

class ChatGroupPage extends StatefulWidget {
  Color primaryColor;
  Color secondaryColor;
  Color chatHeadColor;
  double bubbleRadious;
  bool showArrow;
  String jobName;
  Function dispatcher;

  @override
  _ChatGroupPageState createState() => _ChatGroupPageState();

  ChatGroupPage({this.primaryColor, this.secondaryColor, this.bubbleRadious, this.chatHeadColor, this.showArrow, this.dispatcher, this.jobName});
}

class _ChatGroupPageState extends State<ChatGroupPage> {
  DataSourceRF dataSource;
  int listType = 1;
  var userid;

  @override
  void initState() {
    print('CHAT GROUP PAGE INIT');
    rf.onMessageReceive = (data) {
      dataSource.modelList.clear();
      setState(() {});
    };

    Api.onFetchCompleted = (model, count) {
      if (model == 'all fetch completed') {
        dataSource.modelList.clear();
        print("Printing on Complete: " + model + " " + count.toString());
        setState(() {});
      }
    };

    dataSource = DataSourceRF(modelList: List<ChatGroup>(), type: DataSourceType.local, orderField: 'SendAtTime');
    super.initState();
  }

  @override
  void didChangeDependencies() async {
    print('CHAT GROUP PAGE DIDCHANGEDIPENDENCY');
    userid = await rf.pref.getString('UserGeoId');
    /* if (widget.jobName.isNotEmpty && widget.dispatcher != null)
      rf.api.fetchModel(context: context, serviceName: widget.jobName, dispatcher: widget.dispatcher, modelsName: 'ChatGroup', showDialog: false, showNotification: false);
*/
  }

  @override
  void dispose() {
    Api.onFetchCompleted = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    print('CHAT GROUP PAGE BUILD');
    return Scaffold(
      key: GlobalKey(),
      appBar: AppBar(
        automaticallyImplyLeading: false,
        backgroundColor: Colors.grey[100],
        title: Container(
          child: Row(
            mainAxisSize: MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              InkWell(
                onTap: () => rf.page.back(context), //rf.page.back(_context),
                child: Container(margin: const EdgeInsets.all(5), child: Icon(Icons.arrow_back_ios, color: Colors.black)),
              ),
              SizedBox(
                width: 10,
              ),
              InkWell(
                onTap: () {
                  listType = 1;
                  dataSource = DataSourceRF(modelList: List<ChatGroup>(), type: DataSourceType.local, orderField: 'SendAtTime');
                  setState(() {});
                },
                child: Container(
                  padding: const EdgeInsets.only(left: 12, right: 12, top: 5, bottom: 5),
                  decoration: BoxDecoration(
                    color: listType == 1 ? widget.primaryColor ?? Colors.blue.withOpacity(0.7) : Colors.white,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    border: Border.all(color: listType == 1 ? Colors.black : widget.primaryColor ?? Colors.blue),
                  ),
                  child: TextRF(
                    'ChatList',
                    size: 18,
                    overflow: TextOverflow.ellipsis,
                    color: listType == 1 ? Colors.white : Colors.black,
                    weight: FontWeight.normal,
                  ),
                ),
              ),
              SizedBox(
                width: 10,
              ),
              InkWell(
                onTap: () {
                  listType = 2;
                  dataSource = DataSourceRF(modelList: List<ChatUsers>(), type: DataSourceType.local, orderField: 'UserName');
                  setState(() {});
                },
                child: Container(
                  padding: const EdgeInsets.only(left: 12, right: 12, top: 5, bottom: 5),
                  decoration: BoxDecoration(
                    color: listType == 2 ? widget.primaryColor ?? Colors.blue.withOpacity(0.7) : Colors.white,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    border: Border.all(color: listType == 2 ? Colors.black : widget.primaryColor ?? Colors.blue),
                  ),
                  child: TextRF(
                    'Contacts',
                    size: 18,
                    overflow: TextOverflow.ellipsis,
                    color: listType == 2 ? Colors.white : Colors.black,
                    weight: FontWeight.normal,
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
      body: body(),
    );
  }

  body() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Container(
          height: ScreenRF.height(context) - 150,
          child: ListViewRF(
            dataSource: dataSource,
            builder: (model, index) => listType == 1 ? conversationCard(model, index) : userCard(model, index),
          ),
        ),
      ],
    );
  }

  conversationCard(modelm, index) {
    ChatGroup model = modelm;
    return InkWell(
      onTap: () async {
        rf.onMessageReceive = null;
        rf.chat.callChatPage(model, userid, replace: true);
      },
      child: Container(
        width: ScreenRF.width(context),
        margin: const EdgeInsets.all(10),
        padding: const EdgeInsets.all(10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(0)),
          boxShadow: <BoxShadow>[
            BoxShadow(
              offset: Offset(0, 0),
              blurRadius: 5,
              color: Colors.grey.withOpacity(.15),
            )
          ],
        ),
        child: Stack(
          children: [
            Row(
              mainAxisSize: MainAxisSize.max,
              children: [
                Container(
                  height: 60,
                  width: 60,
                  margin: const EdgeInsets.all(12),
                  decoration: BoxDecoration(
                    color: widget.chatHeadColor ?? Colors.deepPurple,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    boxShadow: <BoxShadow>[
                      BoxShadow(
                        offset: Offset(0, 0),
                        blurRadius: 5,
                        color: widget.chatHeadColor.withOpacity(0.5) ?? Colors.deepPurple.withOpacity(.5),
                      )
                    ],
                  ),
                  child: Center(
                      child: TextRF(
                    UtilRF.getInitials(model.Name),
                    size: 20,
                    color: Colors.white,
                    weight: FontWeight.bold,
                  )),
                ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      GapRF(5),
                      Container(
                        width: 250,
                        child: TextRF(
                          model.Name,
                          size: 18,
                          overflow: TextOverflow.ellipsis,
                          color: Colors.black,
                          weight: FontWeight.bold,
                        ),
                      ),
                      GapRF(5),
                      TextRF(
                        model.Message ?? 'no message sent yet',
                        size: 13,
                        color: Colors.grey,
                        weight: FontWeight.w400,
                      )
                    ],
                  ),
                ),
              ],
            ),
            Positioned(
              top: 10,
              right: 0,
              child: Container(
                width: 80,
                child: TextRF(
                  model.SendAt ?? '',
                  size: 10,
                  overflow: TextOverflow.ellipsis,
                  color: Colors.grey,
                  weight: FontWeight.w400,
                ),
              ),
            )
          ],
        ),
      ),
    );
  }

  userCard(modelm, index) {
    ChatUsers model = modelm;
    return InkWell(
      onTap: () {
        rf.chat.createGroup(user: model).then((value) {
          rf.chat.callChatPage(value, userid, replace: true);
        });
      },
      child: Container(
        width: ScreenRF.width(context),
        margin: const EdgeInsets.all(10),
        padding: const EdgeInsets.all(10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(0)),
          boxShadow: <BoxShadow>[
            BoxShadow(
              offset: Offset(0, 0),
              blurRadius: 5,
              color: Colors.grey.withOpacity(.15),
            )
          ],
        ),
        child: Stack(
          children: [
            Row(
              mainAxisSize: MainAxisSize.max,
              children: [
                Container(
                  height: 60,
                  width: 60,
                  margin: const EdgeInsets.all(12),
                  decoration: BoxDecoration(
                    color: widget.chatHeadColor ?? Colors.deepPurple,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    boxShadow: <BoxShadow>[
                      BoxShadow(
                        offset: Offset(0, 0),
                        blurRadius: 5,
                        color: widget.chatHeadColor.withOpacity(0.5) ?? Colors.deepPurple.withOpacity(.5),
                      )
                    ],
                  ),
                  child: Center(
                      child: TextRF(
                    UtilRF.getInitials(model.UserName),
                    size: 20,
                    color: Colors.white,
                    weight: FontWeight.bold,
                  )),
                ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      GapRF(5),
                      Container(
                        width: 250,
                        child: TextRF(
                          model.UserName,
                          size: 18,
                          overflow: TextOverflow.ellipsis,
                          color: Colors.black,
                          weight: FontWeight.bold,
                        ),
                      ),
                      GapRF(5),
                      TextRF(
                        model.UserTable ?? 'no message sent yet',
                        size: 13,
                        color: Colors.grey,
                        weight: FontWeight.w400,
                      )
                    ],
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}
